from .document import DocumentSet

if False:
    from ..document import Document


class MatchSet(DocumentSet):
    """
    :class:`MatchSet` inherits from :class:`DocumentSet`.
    It's a subset of Documents that represents the matches

    :param docs_proto: Set of matches of the `reference_doc`
    :type docs_proto: :class:`Document`
    :param reference_doc: Reference :class:`Document` for the sub-documents
    :type reference_doc: :class:`Document`
    """

    def __init__(self, docs_proto, reference_doc: 'Document'):
        super().__init__(docs_proto)
        self._ref_doc = reference_doc

    def append(self, document: 'Document', **kwargs) -> 'Document':
        """Add a matched document to the current Document.

        :param document: Sub-document to be added
        :type document: :class: `Document
        :param kwargs: Extra key value arguments
        :return: the newly added sub-document in :class:`Document` view
        :rtype: :class:`Document` view
        """
        from ..document import Document

        m = self._docs_proto.add()
        m.CopyFrom(document.proto)
        match = Document(m)

        match.set_attrs(
            granularity=self.granularity, adjacency=self.adjacency, **kwargs
        )
        match.score.ref_id = self._ref_doc.id

        return match

    @property
    def reference_doc(self) -> 'Document':
        """Get the document that this :class:`MatchSet` referring to.
        :return: the document the match refers to
        """
        return self._ref_doc

    @property
    def granularity(self) -> int:
        """Get granularity of all document in this set.
        :return: the granularity of the documents of which these are match
        """
        return self._ref_doc.granularity

    @property
    def adjacency(self) -> int:
        """Get the adjacency of all document in this set.
        :return: the adjacency of the set of matches
        """
        return self._ref_doc.adjacency + 1



import os
import sys
import warnings
from types import SimpleNamespace, ModuleType
from typing import Optional, List, Any, Dict
from collections import defaultdict

IMPORTED = SimpleNamespace()
IMPORTED.executors = False
IMPORTED.executors = False
IMPORTED.drivers = False
IMPORTED.hub = False
IMPORTED.schema_executors = {}
IMPORTED.schema_drivers = {}


def import_classes(
    namespace: str, show_import_table: bool = False, import_once: bool = False
):
    """
    Import all or selected executors into the runtime. This is called when Jina is first imported for registering the YAML constructor beforehand. It can be also used to import third-part or external executors.

    :param namespace: the namespace to import
    :param show_import_table: show the import result as a table
    :param import_once: import everything only once, to avoid repeated import
    :return: the dependency tree of the imported classes under the `namespace`
    """
    _namespace2type = {
        'jina.executors': 'ExecutorType',
        'jina.drivers': 'DriverType',
        'jina.hub': 'ExecutorType',
    }
    _import_type = _namespace2type.get(namespace)
    if _import_type is None:
        raise TypeError(f'namespace: {namespace} is unrecognized')

    _imported_property = namespace.split('.')[-1]
    _is_imported = getattr(IMPORTED, _imported_property)
    if import_once and _is_imported:
        warnings.warn(
            f'{namespace} has already imported. If you want to re-imported, please set `import_once=True`',
            ImportWarning,
        )
        return {}

    try:
        _modules = _get_modules(namespace)
    except ImportError as e:
        warnings.warn(f'{namespace} has no module to import. {e}', ImportWarning)
        return {}

    load_stat = defaultdict(list)
    bad_imports = []
    depend_tree = {}
    for _mod_name in _modules:
        try:
            bad_imports += _import_module(
                _mod_name, _import_type, depend_tree, load_stat
            )
        except Exception as ex:
            load_stat[_mod_name].append(('', False, ex))
            bad_imports.append(_mod_name)

    if show_import_table:
        _print_load_table(load_stat)
    else:
        _raise_bad_imports_warnings(bad_imports, namespace)

    setattr(IMPORTED, _imported_property, True)

    return depend_tree


class ImportExtensions:
    """
    A context manager for wrapping extension import and fallback. It guides the user to pip install correct package by looking up extra-requirements.txt.

    :param required: set to True if you want to raise the ModuleNotFound error
    :param logger: when not given, built-in warnings.warn will be used
    :param help_text: the help text followed after
    :param pkg_name: the package name to find in extra_requirements.txt, when not given the ModuleNotFound exec_val will be used as the best guess
    """

    def __init__(
        self,
        required: bool,
        logger=None,
        help_text: Optional[str] = None,
        pkg_name: Optional[str] = None,
        verbose: bool = True,
    ):
        self._required = required
        self._tags = []
        self._help_text = help_text
        self._logger = logger
        self._pkg_name = pkg_name
        self._verbose = verbose

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, traceback):
        if exc_type == ModuleNotFoundError:
            missing_module = self._pkg_name or exc_val.name
            from pkg_resources import resource_filename

            with open(
                resource_filename(
                    'jina', '/'.join(('resources', 'extra-requirements.txt'))
                )
            ) as fp:
                for v in fp:
                    if (
                        v.strip()
                        and not v.startswith('#')
                        and v.startswith(missing_module)
                        and ':' in v
                    ):
                        missing_module, install_tags = v.split(':')
                        self._tags.append(missing_module)
                        self._tags.extend(vv.strip() for vv in install_tags.split(','))
                        break

            if self._tags:
                req_msg = 'fallback to default behavior'
                if self._required:
                    req_msg = 'and it is required'
                err_msg = (
                    f'Module "{missing_module}" is not installed, {req_msg}. '
                    f'You are trying to use an extension feature not enabled by the '
                    'current installation.\n'
                    'This feature is available in: '
                )
                from .helper import colored

                err_msg += ' '.join(
                    colored(f'[{tag}]', attrs='bold') for tag in self._tags
                )
                err_msg += f'\nUse {colored("pip install jina[TAG]", attrs="bold")} to enable it'

            else:
                err_msg = f'{exc_val.msg}'

            if self._required:
                if self._verbose:
                    if self._logger:
                        self._logger.critical(err_msg)
                        if self._help_text:
                            self._logger.error(self._help_text)
                    else:
                        warnings.warn(err_msg, RuntimeWarning, stacklevel=2)
                raise exc_val
            else:
                if self._verbose:
                    if self._logger:
                        self._logger.warning(err_msg)
                        if self._help_text:
                            self._logger.info(self._help_text)
                    else:
                        warnings.warn(err_msg, RuntimeWarning, stacklevel=2)
                return True  # suppress the error


class PathImporter:
    """The class to import modules from paths."""

    @staticmethod
    def add_modules(*paths) -> Optional[ModuleType]:
        """
        Import modules from paths.

        :param paths: Paths of the modules.
        :return: The target module.
        """
        for p in paths:
            if not os.path.exists(p):
                raise FileNotFoundError(
                    f'cannot import module from {p}, file not exist'
                )
            module = PathImporter._path_import(p)
        return module

    @staticmethod
    def _path_import(absolute_path: str) -> Optional[ModuleType]:
        import importlib.util

        try:
            # I dont want to trust user path based on directory structure, "jinahub", period
            spec = importlib.util.spec_from_file_location('jinahub', absolute_path)
            module = importlib.util.module_from_spec(spec)
            user_module_name = os.path.splitext(os.path.basename(absolute_path))[0]
            if user_module_name == '__init__':
                # __init__ can not be used as a module name
                spec_name = spec.name
            else:
                spec_name = f'{spec.name}.{user_module_name}'
            sys.modules[spec_name] = module
            spec.loader.exec_module(module)
        except Exception as ex:
            raise ImportError(f'can not import module from {absolute_path}') from ex
        return module


def _print_load_table(load_stat: Dict[str, List[Any]], logger=None):
    from .helper import colored

    load_table = []
    cached = set()

    for k, v in load_stat.items():
        for cls_name, import_stat, err_reason in v:
            if cls_name not in cached:
                load_table.append(
                    f'{colored("✓", "green") if import_stat else colored("✗", "red"):<5} {cls_name if cls_name else colored("Module load error", "red"):<25} {k:<40} {str(err_reason)}'
                )
                cached.add(cls_name)
    if load_table:
        load_table.sort()
        load_table = [
            '',
            '%-5s %-25s %-40s %-s' % ('Load', 'Class', 'Module', 'Dependency'),
            '%-5s %-25s %-40s %-s' % ('-' * 5, '-' * 25, '-' * 40, '-' * 10),
        ] + load_table
        pr = logger.info if logger else print
        pr('\n'.join(load_table))


def _print_dep_tree_rst(fp, dep_tree, title='Executor'):
    tableview = set()
    treeview = []

    def _iter(d, depth):
        for k, v in d.items():
            if k != 'module':
                treeview.append('   ' * depth + f'- `{k}`')
                tableview.add(
                    f'| `{k}` | '
                    + (f'`{d["module"]}`' if 'module' in d else ' ')
                    + ' |'
                )
                _iter(v, depth + 1)

    _iter(dep_tree, 0)

    fp.write(
        f'# List of {len(tableview)} {title}s in Jina\n\n'
        f'This version of Jina includes {len(tableview)} {title}s.\n\n'
        f'## Inheritances in a Tree View\n'
    )
    fp.write('\n'.join(treeview))

    fp.write(f'\n\n## Modules in a Table View \n\n| Class | Module |\n')
    fp.write('| --- | --- |\n')
    fp.write('\n'.join(sorted(tableview)))


def _raise_bad_imports_warnings(bad_imports, namespace):
    if not bad_imports:
        return
    if namespace != 'jina.hub':
        warnings.warn(
            f'theses modules or classes can not be imported {bad_imports}. '
            f'You can use `jina check` to list all executors and drivers'
        )
    else:
        warnings.warn(
            f'due to the missing dependencies or bad implementations, '
            f'{bad_imports} can not be imported '
            f'if you are using these executors/drivers, they wont work. '
            f'You can use `jina check` to list all executors and drivers'
        )


def _get_modules(namespace):
    from setuptools import find_packages
    from pkgutil import get_loader

    try:
        _path = os.path.dirname(get_loader(namespace).path)
    except AttributeError as ex:
        if namespace == 'jina.hub':
            warnings.warn(
                f'hub submodule is not initialized. Please try "git submodule update --init"',
                ImportWarning,
            )
        raise ImportError(f'{namespace} can not be imported. {ex}')

    _modules = _get_submodules(_path, namespace)

    for _pkg in find_packages(_path):
        _modules.add('.'.join([namespace, _pkg]))
        _pkgpath = os.path.join(_path, _pkg.replace('.', '/'))
        _modules |= _get_submodules(_pkgpath, namespace, prefix=_pkg)

    return _filter_modules(_modules)


def _get_submodules(path, namespace, prefix=None):
    from pkgutil import iter_modules

    _prefix = '.'.join([namespace, prefix]) if prefix else namespace
    modules = set()
    for _info in iter_modules([path]):
        _is_hub_module = namespace == 'jina.hub' and _info.ispkg
        _is_nonhub_module = namespace != 'jina.hub' and not _info.ispkg
        module_name = '.'.join([_prefix, _info.name])
        if _is_hub_module or _is_nonhub_module:
            modules.add(module_name)
    return modules


def _filter_modules(modules):
    import re

    _ignored_module_pattern = re.compile(r'\.tests|\.api|\.bump_version')
    return {m for m in modules if not _ignored_module_pattern.findall(m)}


def _load_default_exc_config(cls_obj):
    from .executors.requests import get_default_reqs

    try:
        _request = get_default_reqs(type.mro(cls_obj))
    except ValueError as ex:
        warnings.warn(
            f'Please ensure a config yml is given for {cls_obj.__name__}. {ex}'
        )


def _update_depend_tree(cls_obj, module_name, cur_tree):
    d = cur_tree
    for vvv in cls_obj.mro()[:-1][::-1]:
        if vvv.__name__ not in d:
            d[vvv.__name__] = {}
        d = d[vvv.__name__]
    d['module'] = module_name


def _import_module(module_name, import_type, depend_tree, load_stat):
    from importlib import import_module
    from .helper import colored
    from .schemas.helper import _jina_class_to_schema

    bad_imports = []
    _mod_obj = import_module(module_name)
    for _attr in dir(_mod_obj):
        _cls_obj = getattr(_mod_obj, _attr)
        if _cls_obj.__class__.__name__ != import_type:
            continue
        # update the dependency tree for each class
        try:
            _update_depend_tree(_cls_obj, module_name, depend_tree)
            if _cls_obj.__class__.__name__ == 'ExecutorType':
                _load_default_exc_config(_cls_obj)
                IMPORTED.schema_executors[
                    f'Jina::Executors::{_cls_obj.__name__}'
                ] = _jina_class_to_schema(_cls_obj)
            else:
                IMPORTED.schema_drivers[
                    f'Jina::Drivers::{_cls_obj.__name__}'
                ] = _jina_class_to_schema(_cls_obj)
            # TODO: _success_msg is never used
            _success_msg = colored('▸', 'green').join(
                f'{vvv.__name__}' for vvv in _cls_obj.mro()[:-1][::-1]
            )
            load_stat[module_name].append((_attr, True, _success_msg))
        except Exception as ex:
            load_stat[module_name].append((_attr, False, ex))
            bad_imports.append('.'.join([module_name, _attr]))
    return bad_imports



__copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
__license__ = "Apache-2.0"


def _update_autocomplete():
    from jina.parsers import get_main_parser

    def _gaa(key, parser):
        _result = {}
        _compl = []
        for v in parser._actions:
            if v.option_strings:
                _compl.extend(v.option_strings)
            elif v.choices:
                _compl.extend(v.choices)
                for kk, vv in v.choices.items():
                    _result.update(_gaa(" ".join([key, kk]).strip(), vv))
        # filer out single dash, as they serve as abbrev
        _compl = [k for k in _compl if (not k.startswith("-") or k.startswith("--"))]
        _result.update({key: _compl})
        return _result

    compl = _gaa("", get_main_parser())
    cmd = compl.pop("")
    compl = {"commands": cmd, "completions": compl}

    with open(__file__, "a") as fp:
        fp.write(f"\nac_table = {compl}\n")


if __name__ == "__main__":
    _update_autocomplete()

ac_table = {
    "commands": [
        "--help",
        "--version",
        "--version-full",
        "hello",
        "pod",
        "flow",
        "optimizer",
        "gateway",
        "ping",
        "check",
        "hub",
        "pea",
        "client",
        "export-api",
        "hello-world",
    ],
    "completions": {
        "hello fashion": [
            "--help",
            "--workdir",
            "--download-proxy",
            "--shards",
            "--parallel",
            "--uses-index",
            "--index-data-url",
            "--index-labels-url",
            "--index-request-size",
            "--uses-query",
            "--query-data-url",
            "--query-labels-url",
            "--query-request-size",
            "--num-query",
            "--top-k",
        ],
        "hello chatbot": [
            "--help",
            "--workdir",
            "--download-proxy",
            "--uses",
            "--index-data-url",
            "--demo-url",
            "--port-expose",
            "--parallel",
            "--unblock-query-flow",
        ],
        "hello multimodal": [
            "--help",
            "--workdir",
            "--download-proxy",
            "--uses",
            "--index-data-url",
            "--demo-url",
            "--port-expose",
            "--unblock-query-flow",
        ],
        "hello": ["--help", "fashion", "chatbot", "multimodal"],
        "pod": [
            "--help",
            "--name",
            "--description",
            "--log-config",
            "--quiet",
            "--quiet-error",
            "--identity",
            "--port-ctrl",
            "--ctrl-with-ipc",
            "--timeout-ctrl",
            "--ssh-server",
            "--ssh-keyfile",
            "--ssh-password",
            "--uses",
            "--py-modules",
            "--port-in",
            "--port-out",
            "--host-in",
            "--host-out",
            "--socket-in",
            "--socket-out",
            "--dump-interval",
            "--read-only",
            "--memory-hwm",
            "--on-error-strategy",
            "--num-part",
            "--uses-internal",
            "--entrypoint",
            "--docker-kwargs",
            "--pull-latest",
            "--volumes",
            "--host",
            "--port-expose",
            "--quiet-remote-logs",
            "--upload-files",
            "--workspace-id",
            "--daemon",
            "--runtime-backend",
            "--runtime",
            "--runtime-cls",
            "--timeout-ready",
            "--env",
            "--expose-public",
            "--pea-id",
            "--pea-role",
            "--noblock-on-start",
            "--uses-before",
            "--uses-after",
            "--parallel",
            "--shards",
            "--polling",
            "--scheduling",
            "--pod-role",
        ],
        "flow": [
            "--help",
            "--name",
            "--description",
            "--log-config",
            "--quiet",
            "--quiet-error",
            "--identity",
            "--uses",
            "--inspect",
        ],
        "optimizer": [
            "--help",
            "--name",
            "--description",
            "--log-config",
            "--quiet",
            "--quiet-error",
            "--identity",
            "--uses",
            "--output-dir",
        ],
        "gateway": [
            "--help",
            "--name",
            "--description",
            "--log-config",
            "--quiet",
            "--quiet-error",
            "--identity",
            "--port-ctrl",
            "--ctrl-with-ipc",
            "--timeout-ctrl",
            "--ssh-server",
            "--ssh-keyfile",
            "--ssh-password",
            "--uses",
            "--py-modules",
            "--port-in",
            "--port-out",
            "--host-in",
            "--host-out",
            "--socket-in",
            "--socket-out",
            "--dump-interval",
            "--read-only",
            "--memory-hwm",
            "--on-error-strategy",
            "--num-part",
            "--max-message-size",
            "--proxy",
            "--prefetch",
            "--prefetch-on-recv",
            "--restful",
            "--rest-api",
            "--compress",
            "--compress-min-bytes",
            "--compress-min-ratio",
            "--host",
            "--port-expose",
            "--daemon",
            "--runtime-backend",
            "--runtime",
            "--runtime-cls",
            "--timeout-ready",
            "--env",
            "--expose-public",
            "--pea-id",
            "--pea-role",
            "--noblock-on-start",
        ],
        "ping": ["--help", "--timeout", "--retries", "--print-response"],
        "check": ["--help", "--summary-exec", "--summary-driver"],
        "hub login": ["--help"],
        "hub new": ["--help", "--output-dir", "--template", "--type", "--overwrite"],
        "hub init": ["--help", "--output-dir", "--template", "--type", "--overwrite"],
        "hub create": ["--help", "--output-dir", "--template", "--type", "--overwrite"],
        "hub build": [
            "--help",
            "--username",
            "--password",
            "--registry",
            "--repository",
            "--file",
            "--pull",
            "--push",
            "--dry-run",
            "--prune-images",
            "--raise-error",
            "--test-uses",
            "--test-level",
            "--timeout-ready",
            "--host-info",
            "--daemon",
            "--no-overwrite",
        ],
        "hub push": [
            "--help",
            "--username",
            "--password",
            "--registry",
            "--repository",
            "--no-overwrite",
        ],
        "hub pull": [
            "--help",
            "--username",
            "--password",
            "--registry",
            "--repository",
            "--no-overwrite",
        ],
        "hub list": [
            "--help",
            "--name",
            "--kind",
            "--keywords",
            "--type",
            "--local-only",
        ],
        "hub": [
            "--help",
            "login",
            "new",
            "init",
            "create",
            "build",
            "push",
            "pull",
            "list",
        ],
        "pea": [
            "--help",
            "--name",
            "--description",
            "--log-config",
            "--quiet",
            "--quiet-error",
            "--identity",
            "--port-ctrl",
            "--ctrl-with-ipc",
            "--timeout-ctrl",
            "--ssh-server",
            "--ssh-keyfile",
            "--ssh-password",
            "--uses",
            "--py-modules",
            "--port-in",
            "--port-out",
            "--host-in",
            "--host-out",
            "--socket-in",
            "--socket-out",
            "--dump-interval",
            "--read-only",
            "--memory-hwm",
            "--on-error-strategy",
            "--num-part",
            "--uses-internal",
            "--entrypoint",
            "--docker-kwargs",
            "--pull-latest",
            "--volumes",
            "--host",
            "--port-expose",
            "--quiet-remote-logs",
            "--upload-files",
            "--workspace-id",
            "--daemon",
            "--runtime-backend",
            "--runtime",
            "--runtime-cls",
            "--timeout-ready",
            "--env",
            "--expose-public",
            "--pea-id",
            "--pea-role",
            "--noblock-on-start",
        ],
        "client": [
            "--help",
            "--request-size",
            "--mode",
            "--top-k",
            "--mime-type",
            "--continue-on-error",
            "--return-results",
            "--max-message-size",
            "--proxy",
            "--prefetch",
            "--prefetch-on-recv",
            "--restful",
            "--rest-api",
            "--compress",
            "--compress-min-bytes",
            "--compress-min-ratio",
            "--host",
            "--port-expose",
        ],
        "export-api": ["--help", "--yaml-path", "--json-path", "--schema-path"],
        "hello-world": [
            "--help",
            "--workdir",
            "--download-proxy",
            "--shards",
            "--parallel",
            "--uses-index",
            "--index-data-url",
            "--index-labels-url",
            "--index-request-size",
            "--uses-query",
            "--query-data-url",
            "--query-labels-url",
            "--query-request-size",
            "--num-query",
            "--top-k",
        ],
    },
}



__copyright__ = "Copyright (c) 2021 Jina AI Limited. All rights reserved."
__license__ = "Apache-2.0"

from typing import Tuple, Dict, Union, Iterable

import numpy as np

from jina.executors.decorators import single
from jina.executors.crafters import BaseCrafter

from .helper import _load_image, _move_channel_axis, _crop_image, _resize_short


class ImageNormalizer(BaseCrafter):
    """
    Normalize the image.

    :class:`ImageNormalizer` works on doc-level,
        it receives values of file names on the
        doc-level and returns image matrix on the chunk-level

    :param target_size: Desired output size. If size is a sequence
        like (h, w), the output size will be matched to this.
        If size is an int, the smaller edge of the image will be matched
        to this number maintaining the aspect ratio.
    :param img_mean: The mean of the images in `RGB` channels.
        Set to `[0.485, 0.456, 0.406]` for the models trained
        on `imagenet` with pytorch backbone.
    :param img_std: the std of the images in `RGB` channels.
        Set to `[0.229, 0.224, 0.225]` for the models trained
        on `imagenet` with pytorch backbone.
    :param resize_dim: the size of images' height and width to be resized to.
        The images are resized before cropping to the output size
    :param channel_axis: the axis id of the color channel,
        ``-1`` indicates the color channel info at the last axis
    """

    def __init__(self,
                 target_size: Union[Iterable[int], int] = 224,
                 img_mean: Tuple[float] = (0, 0, 0),
                 img_std: Tuple[float] = (1, 1, 1),
                 resize_dim: int = 256,
                 channel_axis: int = -1,
                 *args,
                 **kwargs):
        """Set Constructor."""
        super().__init__(*args, **kwargs)
        if isinstance(target_size, int):
            self.target_size = target_size
        elif isinstance(target_size, Iterable):
            self.target_size = tuple(target_size)
        else:
            raise ValueError(f'target_size {target_size} should be an integer or tuple/list of 2 integers')
        self.resize_dim = resize_dim
        self.img_mean = np.array(img_mean).reshape((1, 1, 3))
        self.img_std = np.array(img_std).reshape((1, 1, 3))
        self.channel_axis = channel_axis

    @single
    def craft(self, blob: 'np.ndarray', *args, **kwargs) -> Dict:
        """
        Normalize the image.

        :param blob: the ndarray of the image with the color channel at the last axis
        :return: a chunk dict with the normalized image
        """
        raw_img = _load_image(blob, self.channel_axis)
        _img = self._normalize(raw_img)
        img = _move_channel_axis(_img, -1, self.channel_axis)
        return dict(offset=0, blob=img)

    def _normalize(self, img):
        img = _resize_short(img, target_size=self.resize_dim)
        img, _, _ = _crop_image(img, target_size=self.target_size, how='center')
        img = np.array(img).astype('float32')/255
        img -= self.img_mean
        img /= self.img_std
        return img



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: jina.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
    name='jina.proto',
    package='jina',
    syntax='proto3',
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\njina.proto\x12\x04jina\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1cgoogle/protobuf/struct.proto\"\xfd\x01\n\x11\x44\x65nseNdArrayProto\x12\x0e\n\x06\x62uffer\x18\x01 \x01(\x0c\x12\r\n\x05shape\x18\x02 \x03(\r\x12\r\n\x05\x64type\x18\x03 \x01(\t\x12>\n\x0cquantization\x18\x04 \x01(\x0e\x32(.jina.DenseNdArrayProto.QuantizationMode\x12\x0f\n\x07max_val\x18\x05 \x01(\x02\x12\x0f\n\x07min_val\x18\x06 \x01(\x02\x12\r\n\x05scale\x18\x07 \x01(\x02\x12\x16\n\x0eoriginal_dtype\x18\x08 \x01(\t\"1\n\x10QuantizationMode\x12\x08\n\x04NONE\x10\x00\x12\x08\n\x04\x46P16\x10\x01\x12\t\n\x05UINT8\x10\x02\"o\n\x0cNdArrayProto\x12(\n\x05\x64\x65nse\x18\x01 \x01(\x0b\x32\x17.jina.DenseNdArrayProtoH\x00\x12*\n\x06sparse\x18\x02 \x01(\x0b\x32\x18.jina.SparseNdArrayProtoH\x00\x42\t\n\x07\x63ontent\"|\n\x12SparseNdArrayProto\x12(\n\x07indices\x18\x01 \x01(\x0b\x32\x17.jina.DenseNdArrayProto\x12\'\n\x06values\x18\x02 \x01(\x0b\x32\x17.jina.DenseNdArrayProto\x12\x13\n\x0b\x64\x65nse_shape\x18\x03 \x03(\x03\"\x7f\n\x0fNamedScoreProto\x12\r\n\x05value\x18\x01 \x01(\x02\x12\x0f\n\x07op_name\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\'\n\x08operands\x18\x04 \x03(\x0b\x32\x15.jina.NamedScoreProto\x12\x0e\n\x06ref_id\x18\x05 \x01(\t\"\xb0\x04\n\rDocumentProto\x12\n\n\x02id\x18\x01 \x01(\t\x12\x14\n\x0c\x63ontent_hash\x18\x18 \x01(\t\x12\x13\n\x0bgranularity\x18\x0e \x01(\r\x12\x11\n\tadjacency\x18\x16 \x01(\r\x12\x12\n\nlevel_name\x18\x0f \x01(\t\x12\x11\n\tparent_id\x18\x10 \x01(\t\x12\x10\n\x06\x62uffer\x18\x03 \x01(\x0cH\x00\x12\"\n\x04\x62lob\x18\x0c \x01(\x0b\x32\x12.jina.NdArrayProtoH\x00\x12\x0e\n\x04text\x18\r \x01(\tH\x00\x12#\n\x06\x63hunks\x18\x04 \x03(\x0b\x32\x13.jina.DocumentProto\x12\x0e\n\x06weight\x18\x05 \x01(\x02\x12\x0e\n\x06length\x18\x06 \x01(\r\x12$\n\x07matches\x18\x08 \x03(\x0b\x32\x13.jina.DocumentProto\x12\x11\n\tmime_type\x18\n \x01(\t\x12\x0b\n\x03uri\x18\t \x01(\t\x12%\n\x04tags\x18\x0b \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x10\n\x08location\x18\x11 \x03(\r\x12\x0e\n\x06offset\x18\x12 \x01(\r\x12%\n\tembedding\x18\x13 \x01(\x0b\x32\x12.jina.NdArrayProto\x12$\n\x05score\x18\x14 \x01(\x0b\x32\x15.jina.NamedScoreProto\x12\x10\n\x08modality\x18\x15 \x01(\t\x12*\n\x0b\x65valuations\x18\x17 \x03(\x0b\x32\x15.jina.NamedScoreProtoB\t\n\x07\x63ontent\"\xaa\x01\n\nRouteProto\x12\x0b\n\x03pod\x18\x01 \x01(\t\x12\x0e\n\x06pod_id\x18\x02 \x01(\t\x12.\n\nstart_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08\x65nd_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12!\n\x06status\x18\x05 \x01(\x0b\x32\x11.jina.StatusProto\"\xf6\x03\n\rEnvelopeProto\x12\x11\n\tsender_id\x18\x01 \x01(\t\x12\x13\n\x0breceiver_id\x18\x02 \x01(\t\x12\x12\n\nrequest_id\x18\x03 \x01(\t\x12\x0f\n\x07timeout\x18\x04 \x01(\r\x12 \n\x06routes\x18\x05 \x03(\x0b\x32\x10.jina.RouteProto\x12\x31\n\x07version\x18\x06 \x01(\x0b\x32 .jina.EnvelopeProto.VersionProto\x12!\n\x06status\x18\x07 \x01(\x0b\x32\x11.jina.StatusProto\x12\x14\n\x0crequest_type\x18\t \x01(\t\x12\x15\n\rcheck_version\x18\n \x01(\x08\x12<\n\x0b\x63ompression\x18\x0b \x01(\x0b\x32\'.jina.EnvelopeProto.CompressConfigProto\x1a\x38\n\x0cVersionProto\x12\x0c\n\x04jina\x18\x01 \x01(\t\x12\r\n\x05proto\x18\x02 \x01(\t\x12\x0b\n\x03vcs\x18\x03 \x01(\t\x1a{\n\x13\x43ompressConfigProto\x12\x11\n\talgorithm\x18\x01 \x01(\t\x12\x11\n\tmin_bytes\x18\x02 \x01(\x04\x12\x11\n\tmin_ratio\x18\x03 \x01(\x02\x12+\n\nparameters\x18\x04 \x01(\x0b\x32\x17.google.protobuf.Struct\"\xdf\x02\n\x0bStatusProto\x12*\n\x04\x63ode\x18\x01 \x01(\x0e\x32\x1c.jina.StatusProto.StatusCode\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x33\n\texception\x18\x03 \x01(\x0b\x32 .jina.StatusProto.ExceptionProto\x1a^\n\x0e\x45xceptionProto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04\x61rgs\x18\x02 \x03(\t\x12\x0e\n\x06stacks\x18\x03 \x03(\t\x12\x10\n\x08\x65xecutor\x18\x04 \x01(\t\x12\x0e\n\x06\x64river\x18\x05 \x01(\t\"z\n\nStatusCode\x12\x0b\n\x07SUCCESS\x10\x00\x12\x0b\n\x07PENDING\x10\x01\x12\t\n\x05READY\x10\x02\x12\t\n\x05\x45RROR\x10\x03\x12\x13\n\x0f\x45RROR_DUPLICATE\x10\x04\x12\x14\n\x10\x45RROR_NOTALLOWED\x10\x05\x12\x11\n\rERROR_CHAINED\x10\x06\"Z\n\x0cMessageProto\x12%\n\x08\x65nvelope\x18\x01 \x01(\x0b\x32\x13.jina.EnvelopeProto\x12#\n\x07request\x18\x02 \x01(\x0b\x32\x12.jina.RequestProto\"\xa1\x08\n\x0cRequestProto\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x35\n\x05train\x18\x02 \x01(\x0b\x32$.jina.RequestProto.TrainRequestProtoH\x00\x12\x35\n\x05index\x18\x03 \x01(\x0b\x32$.jina.RequestProto.IndexRequestProtoH\x00\x12\x37\n\x06search\x18\x04 \x01(\x0b\x32%.jina.RequestProto.SearchRequestProtoH\x00\x12\x39\n\x07\x63ontrol\x18\x05 \x01(\x0b\x32&.jina.RequestProto.ControlRequestProtoH\x00\x12\x37\n\x06\x64\x65lete\x18\t \x01(\x0b\x32%.jina.RequestProto.DeleteRequestProtoH\x00\x12\x37\n\x06update\x18\n \x01(\x0b\x32%.jina.RequestProto.UpdateRequestProtoH\x00\x12!\n\x06status\x18\x06 \x01(\x0b\x32\x11.jina.StatusProto\x12 \n\x06routes\x18\x08 \x03(\x0b\x32\x10.jina.RouteProto\x12&\n\x08queryset\x18\x07 \x03(\x0b\x32\x14.jina.QueryLangProto\x1a\x61\n\x11TrainRequestProto\x12!\n\x04\x64ocs\x18\x01 \x03(\x0b\x32\x13.jina.DocumentProto\x12)\n\x0cgroundtruths\x18\x02 \x03(\x0b\x32\x13.jina.DocumentProto\x1a\x61\n\x11IndexRequestProto\x12!\n\x04\x64ocs\x18\x01 \x03(\x0b\x32\x13.jina.DocumentProto\x12)\n\x0cgroundtruths\x18\x02 \x03(\x0b\x32\x13.jina.DocumentProto\x1a\x37\n\x12UpdateRequestProto\x12!\n\x04\x64ocs\x18\x01 \x03(\x0b\x32\x13.jina.DocumentProto\x1a!\n\x12\x44\x65leteRequestProto\x12\x0b\n\x03ids\x18\x01 \x03(\t\x1a\x62\n\x12SearchRequestProto\x12!\n\x04\x64ocs\x18\x01 \x03(\x0b\x32\x13.jina.DocumentProto\x12)\n\x0cgroundtruths\x18\x02 \x03(\x0b\x32\x13.jina.DocumentProto\x1a\xad\x01\n\x13\x43ontrolRequestProto\x12?\n\x07\x63ommand\x18\x01 \x01(\x0e\x32..jina.RequestProto.ControlRequestProto.Command\x12%\n\x04\x61rgs\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\".\n\x07\x43ommand\x12\r\n\tTERMINATE\x10\x00\x12\n\n\x06STATUS\x10\x01\x12\x08\n\x04IDLE\x10\x03\x42\x06\n\x04\x62ody\"o\n\x0eQueryLangProto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12+\n\nparameters\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x10\n\x08\x64isabled\x18\x03 \x01(\x08\x12\x10\n\x08priority\x18\x04 \x01(\x05\x32?\n\x07JinaRPC\x12\x34\n\x04\x43\x61ll\x12\x12.jina.RequestProto\x1a\x12.jina.RequestProto\"\x00(\x01\x30\x01\x62\x06proto3',
    dependencies=[
        google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,
        google_dot_protobuf_dot_struct__pb2.DESCRIPTOR,
    ],
)


_DENSENDARRAYPROTO_QUANTIZATIONMODE = _descriptor.EnumDescriptor(
    name='QuantizationMode',
    full_name='jina.DenseNdArrayProto.QuantizationMode',
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name='NONE',
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name='FP16',
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name='UINT8',
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=288,
    serialized_end=337,
)
_sym_db.RegisterEnumDescriptor(_DENSENDARRAYPROTO_QUANTIZATIONMODE)

_STATUSPROTO_STATUSCODE = _descriptor.EnumDescriptor(
    name='StatusCode',
    full_name='jina.StatusProto.StatusCode',
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name='SUCCESS',
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name='PENDING',
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name='READY',
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name='ERROR',
            index=3,
            number=3,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name='ERROR_DUPLICATE',
            index=4,
            number=4,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name='ERROR_NOTALLOWED',
            index=5,
            number=5,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name='ERROR_CHAINED',
            index=6,
            number=6,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=2178,
    serialized_end=2300,
)
_sym_db.RegisterEnumDescriptor(_STATUSPROTO_STATUSCODE)

_REQUESTPROTO_CONTROLREQUESTPROTO_COMMAND = _descriptor.EnumDescriptor(
    name='Command',
    full_name='jina.RequestProto.ControlRequestProto.Command',
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name='TERMINATE',
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name='STATUS',
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name='IDLE',
            index=2,
            number=3,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=3398,
    serialized_end=3444,
)
_sym_db.RegisterEnumDescriptor(_REQUESTPROTO_CONTROLREQUESTPROTO_COMMAND)


_DENSENDARRAYPROTO = _descriptor.Descriptor(
    name='DenseNdArrayProto',
    full_name='jina.DenseNdArrayProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='buffer',
            full_name='jina.DenseNdArrayProto.buffer',
            index=0,
            number=1,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"",
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='shape',
            full_name='jina.DenseNdArrayProto.shape',
            index=1,
            number=2,
            type=13,
            cpp_type=3,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='dtype',
            full_name='jina.DenseNdArrayProto.dtype',
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='quantization',
            full_name='jina.DenseNdArrayProto.quantization',
            index=3,
            number=4,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='max_val',
            full_name='jina.DenseNdArrayProto.max_val',
            index=4,
            number=5,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='min_val',
            full_name='jina.DenseNdArrayProto.min_val',
            index=5,
            number=6,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='scale',
            full_name='jina.DenseNdArrayProto.scale',
            index=6,
            number=7,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='original_dtype',
            full_name='jina.DenseNdArrayProto.original_dtype',
            index=7,
            number=8,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[
        _DENSENDARRAYPROTO_QUANTIZATIONMODE,
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=84,
    serialized_end=337,
)


_NDARRAYPROTO = _descriptor.Descriptor(
    name='NdArrayProto',
    full_name='jina.NdArrayProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='dense',
            full_name='jina.NdArrayProto.dense',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='sparse',
            full_name='jina.NdArrayProto.sparse',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name='content',
            full_name='jina.NdArrayProto.content',
            index=0,
            containing_type=None,
            create_key=_descriptor._internal_create_key,
            fields=[],
        ),
    ],
    serialized_start=339,
    serialized_end=450,
)


_SPARSENDARRAYPROTO = _descriptor.Descriptor(
    name='SparseNdArrayProto',
    full_name='jina.SparseNdArrayProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='indices',
            full_name='jina.SparseNdArrayProto.indices',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='values',
            full_name='jina.SparseNdArrayProto.values',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='dense_shape',
            full_name='jina.SparseNdArrayProto.dense_shape',
            index=2,
            number=3,
            type=3,
            cpp_type=2,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=452,
    serialized_end=576,
)


_NAMEDSCOREPROTO = _descriptor.Descriptor(
    name='NamedScoreProto',
    full_name='jina.NamedScoreProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='value',
            full_name='jina.NamedScoreProto.value',
            index=0,
            number=1,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='op_name',
            full_name='jina.NamedScoreProto.op_name',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='description',
            full_name='jina.NamedScoreProto.description',
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='operands',
            full_name='jina.NamedScoreProto.operands',
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='ref_id',
            full_name='jina.NamedScoreProto.ref_id',
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=578,
    serialized_end=705,
)


_DOCUMENTPROTO = _descriptor.Descriptor(
    name='DocumentProto',
    full_name='jina.DocumentProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='id',
            full_name='jina.DocumentProto.id',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='content_hash',
            full_name='jina.DocumentProto.content_hash',
            index=1,
            number=24,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='granularity',
            full_name='jina.DocumentProto.granularity',
            index=2,
            number=14,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='adjacency',
            full_name='jina.DocumentProto.adjacency',
            index=3,
            number=22,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='level_name',
            full_name='jina.DocumentProto.level_name',
            index=4,
            number=15,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='parent_id',
            full_name='jina.DocumentProto.parent_id',
            index=5,
            number=16,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='buffer',
            full_name='jina.DocumentProto.buffer',
            index=6,
            number=3,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"",
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='blob',
            full_name='jina.DocumentProto.blob',
            index=7,
            number=12,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='text',
            full_name='jina.DocumentProto.text',
            index=8,
            number=13,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='chunks',
            full_name='jina.DocumentProto.chunks',
            index=9,
            number=4,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='weight',
            full_name='jina.DocumentProto.weight',
            index=10,
            number=5,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='length',
            full_name='jina.DocumentProto.length',
            index=11,
            number=6,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='matches',
            full_name='jina.DocumentProto.matches',
            index=12,
            number=8,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='mime_type',
            full_name='jina.DocumentProto.mime_type',
            index=13,
            number=10,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='uri',
            full_name='jina.DocumentProto.uri',
            index=14,
            number=9,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='tags',
            full_name='jina.DocumentProto.tags',
            index=15,
            number=11,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='location',
            full_name='jina.DocumentProto.location',
            index=16,
            number=17,
            type=13,
            cpp_type=3,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='offset',
            full_name='jina.DocumentProto.offset',
            index=17,
            number=18,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='embedding',
            full_name='jina.DocumentProto.embedding',
            index=18,
            number=19,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='score',
            full_name='jina.DocumentProto.score',
            index=19,
            number=20,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='modality',
            full_name='jina.DocumentProto.modality',
            index=20,
            number=21,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='evaluations',
            full_name='jina.DocumentProto.evaluations',
            index=21,
            number=23,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name='content',
            full_name='jina.DocumentProto.content',
            index=0,
            containing_type=None,
            create_key=_descriptor._internal_create_key,
            fields=[],
        ),
    ],
    serialized_start=708,
    serialized_end=1268,
)


_ROUTEPROTO = _descriptor.Descriptor(
    name='RouteProto',
    full_name='jina.RouteProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='pod',
            full_name='jina.RouteProto.pod',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='pod_id',
            full_name='jina.RouteProto.pod_id',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='start_time',
            full_name='jina.RouteProto.start_time',
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='end_time',
            full_name='jina.RouteProto.end_time',
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='status',
            full_name='jina.RouteProto.status',
            index=4,
            number=5,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1271,
    serialized_end=1441,
)


_ENVELOPEPROTO_VERSIONPROTO = _descriptor.Descriptor(
    name='VersionProto',
    full_name='jina.EnvelopeProto.VersionProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='jina',
            full_name='jina.EnvelopeProto.VersionProto.jina',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='proto',
            full_name='jina.EnvelopeProto.VersionProto.proto',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='vcs',
            full_name='jina.EnvelopeProto.VersionProto.vcs',
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1765,
    serialized_end=1821,
)

_ENVELOPEPROTO_COMPRESSCONFIGPROTO = _descriptor.Descriptor(
    name='CompressConfigProto',
    full_name='jina.EnvelopeProto.CompressConfigProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='algorithm',
            full_name='jina.EnvelopeProto.CompressConfigProto.algorithm',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='min_bytes',
            full_name='jina.EnvelopeProto.CompressConfigProto.min_bytes',
            index=1,
            number=2,
            type=4,
            cpp_type=4,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='min_ratio',
            full_name='jina.EnvelopeProto.CompressConfigProto.min_ratio',
            index=2,
            number=3,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='parameters',
            full_name='jina.EnvelopeProto.CompressConfigProto.parameters',
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1823,
    serialized_end=1946,
)

_ENVELOPEPROTO = _descriptor.Descriptor(
    name='EnvelopeProto',
    full_name='jina.EnvelopeProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='sender_id',
            full_name='jina.EnvelopeProto.sender_id',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='receiver_id',
            full_name='jina.EnvelopeProto.receiver_id',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='request_id',
            full_name='jina.EnvelopeProto.request_id',
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='timeout',
            full_name='jina.EnvelopeProto.timeout',
            index=3,
            number=4,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='routes',
            full_name='jina.EnvelopeProto.routes',
            index=4,
            number=5,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='version',
            full_name='jina.EnvelopeProto.version',
            index=5,
            number=6,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='status',
            full_name='jina.EnvelopeProto.status',
            index=6,
            number=7,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='request_type',
            full_name='jina.EnvelopeProto.request_type',
            index=7,
            number=9,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='check_version',
            full_name='jina.EnvelopeProto.check_version',
            index=8,
            number=10,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='compression',
            full_name='jina.EnvelopeProto.compression',
            index=9,
            number=11,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[
        _ENVELOPEPROTO_VERSIONPROTO,
        _ENVELOPEPROTO_COMPRESSCONFIGPROTO,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1444,
    serialized_end=1946,
)


_STATUSPROTO_EXCEPTIONPROTO = _descriptor.Descriptor(
    name='ExceptionProto',
    full_name='jina.StatusProto.ExceptionProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='name',
            full_name='jina.StatusProto.ExceptionProto.name',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='args',
            full_name='jina.StatusProto.ExceptionProto.args',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='stacks',
            full_name='jina.StatusProto.ExceptionProto.stacks',
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='executor',
            full_name='jina.StatusProto.ExceptionProto.executor',
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='driver',
            full_name='jina.StatusProto.ExceptionProto.driver',
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=2082,
    serialized_end=2176,
)

_STATUSPROTO = _descriptor.Descriptor(
    name='StatusProto',
    full_name='jina.StatusProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='code',
            full_name='jina.StatusProto.code',
            index=0,
            number=1,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='description',
            full_name='jina.StatusProto.description',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='exception',
            full_name='jina.StatusProto.exception',
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[
        _STATUSPROTO_EXCEPTIONPROTO,
    ],
    enum_types=[
        _STATUSPROTO_STATUSCODE,
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=1949,
    serialized_end=2300,
)


_MESSAGEPROTO = _descriptor.Descriptor(
    name='MessageProto',
    full_name='jina.MessageProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='envelope',
            full_name='jina.MessageProto.envelope',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='request',
            full_name='jina.MessageProto.request',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=2302,
    serialized_end=2392,
)


_REQUESTPROTO_TRAINREQUESTPROTO = _descriptor.Descriptor(
    name='TrainRequestProto',
    full_name='jina.RequestProto.TrainRequestProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='docs',
            full_name='jina.RequestProto.TrainRequestProto.docs',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='groundtruths',
            full_name='jina.RequestProto.TrainRequestProto.groundtruths',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=2880,
    serialized_end=2977,
)

_REQUESTPROTO_INDEXREQUESTPROTO = _descriptor.Descriptor(
    name='IndexRequestProto',
    full_name='jina.RequestProto.IndexRequestProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='docs',
            full_name='jina.RequestProto.IndexRequestProto.docs',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='groundtruths',
            full_name='jina.RequestProto.IndexRequestProto.groundtruths',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=2979,
    serialized_end=3076,
)

_REQUESTPROTO_UPDATEREQUESTPROTO = _descriptor.Descriptor(
    name='UpdateRequestProto',
    full_name='jina.RequestProto.UpdateRequestProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='docs',
            full_name='jina.RequestProto.UpdateRequestProto.docs',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=3078,
    serialized_end=3133,
)

_REQUESTPROTO_DELETEREQUESTPROTO = _descriptor.Descriptor(
    name='DeleteRequestProto',
    full_name='jina.RequestProto.DeleteRequestProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='ids',
            full_name='jina.RequestProto.DeleteRequestProto.ids',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=3135,
    serialized_end=3168,
)

_REQUESTPROTO_SEARCHREQUESTPROTO = _descriptor.Descriptor(
    name='SearchRequestProto',
    full_name='jina.RequestProto.SearchRequestProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='docs',
            full_name='jina.RequestProto.SearchRequestProto.docs',
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='groundtruths',
            full_name='jina.RequestProto.SearchRequestProto.groundtruths',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=3170,
    serialized_end=3268,
)

_REQUESTPROTO_CONTROLREQUESTPROTO = _descriptor.Descriptor(
    name='ControlRequestProto',
    full_name='jina.RequestProto.ControlRequestProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='command',
            full_name='jina.RequestProto.ControlRequestProto.command',
            index=0,
            number=1,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='args',
            full_name='jina.RequestProto.ControlRequestProto.args',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[
        _REQUESTPROTO_CONTROLREQUESTPROTO_COMMAND,
    ],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=3271,
    serialized_end=3444,
)

_REQUESTPROTO = _descriptor.Descriptor(
    name='RequestProto',
    full_name='jina.RequestProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='request_id',
            full_name='jina.RequestProto.request_id',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='train',
            full_name='jina.RequestProto.train',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='index',
            full_name='jina.RequestProto.index',
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='search',
            full_name='jina.RequestProto.search',
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='control',
            full_name='jina.RequestProto.control',
            index=4,
            number=5,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='delete',
            full_name='jina.RequestProto.delete',
            index=5,
            number=9,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='update',
            full_name='jina.RequestProto.update',
            index=6,
            number=10,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='status',
            full_name='jina.RequestProto.status',
            index=7,
            number=6,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='routes',
            full_name='jina.RequestProto.routes',
            index=8,
            number=8,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='queryset',
            full_name='jina.RequestProto.queryset',
            index=9,
            number=7,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[
        _REQUESTPROTO_TRAINREQUESTPROTO,
        _REQUESTPROTO_INDEXREQUESTPROTO,
        _REQUESTPROTO_UPDATEREQUESTPROTO,
        _REQUESTPROTO_DELETEREQUESTPROTO,
        _REQUESTPROTO_SEARCHREQUESTPROTO,
        _REQUESTPROTO_CONTROLREQUESTPROTO,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name='body',
            full_name='jina.RequestProto.body',
            index=0,
            containing_type=None,
            create_key=_descriptor._internal_create_key,
            fields=[],
        ),
    ],
    serialized_start=2395,
    serialized_end=3452,
)


_QUERYLANGPROTO = _descriptor.Descriptor(
    name='QueryLangProto',
    full_name='jina.QueryLangProto',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='name',
            full_name='jina.QueryLangProto.name',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='parameters',
            full_name='jina.QueryLangProto.parameters',
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='disabled',
            full_name='jina.QueryLangProto.disabled',
            index=2,
            number=3,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name='priority',
            full_name='jina.QueryLangProto.priority',
            index=3,
            number=4,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=3454,
    serialized_end=3565,
)

_DENSENDARRAYPROTO.fields_by_name[
    'quantization'
].enum_type = _DENSENDARRAYPROTO_QUANTIZATIONMODE
_DENSENDARRAYPROTO_QUANTIZATIONMODE.containing_type = _DENSENDARRAYPROTO
_NDARRAYPROTO.fields_by_name['dense'].message_type = _DENSENDARRAYPROTO
_NDARRAYPROTO.fields_by_name['sparse'].message_type = _SPARSENDARRAYPROTO
_NDARRAYPROTO.oneofs_by_name['content'].fields.append(
    _NDARRAYPROTO.fields_by_name['dense']
)
_NDARRAYPROTO.fields_by_name['dense'].containing_oneof = _NDARRAYPROTO.oneofs_by_name[
    'content'
]
_NDARRAYPROTO.oneofs_by_name['content'].fields.append(
    _NDARRAYPROTO.fields_by_name['sparse']
)
_NDARRAYPROTO.fields_by_name['sparse'].containing_oneof = _NDARRAYPROTO.oneofs_by_name[
    'content'
]
_SPARSENDARRAYPROTO.fields_by_name['indices'].message_type = _DENSENDARRAYPROTO
_SPARSENDARRAYPROTO.fields_by_name['values'].message_type = _DENSENDARRAYPROTO
_NAMEDSCOREPROTO.fields_by_name['operands'].message_type = _NAMEDSCOREPROTO
_DOCUMENTPROTO.fields_by_name['blob'].message_type = _NDARRAYPROTO
_DOCUMENTPROTO.fields_by_name['chunks'].message_type = _DOCUMENTPROTO
_DOCUMENTPROTO.fields_by_name['matches'].message_type = _DOCUMENTPROTO
_DOCUMENTPROTO.fields_by_name[
    'tags'
].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_DOCUMENTPROTO.fields_by_name['embedding'].message_type = _NDARRAYPROTO
_DOCUMENTPROTO.fields_by_name['score'].message_type = _NAMEDSCOREPROTO
_DOCUMENTPROTO.fields_by_name['evaluations'].message_type = _NAMEDSCOREPROTO
_DOCUMENTPROTO.oneofs_by_name['content'].fields.append(
    _DOCUMENTPROTO.fields_by_name['buffer']
)
_DOCUMENTPROTO.fields_by_name[
    'buffer'
].containing_oneof = _DOCUMENTPROTO.oneofs_by_name['content']
_DOCUMENTPROTO.oneofs_by_name['content'].fields.append(
    _DOCUMENTPROTO.fields_by_name['blob']
)
_DOCUMENTPROTO.fields_by_name['blob'].containing_oneof = _DOCUMENTPROTO.oneofs_by_name[
    'content'
]
_DOCUMENTPROTO.oneofs_by_name['content'].fields.append(
    _DOCUMENTPROTO.fields_by_name['text']
)
_DOCUMENTPROTO.fields_by_name['text'].containing_oneof = _DOCUMENTPROTO.oneofs_by_name[
    'content'
]
_ROUTEPROTO.fields_by_name[
    'start_time'
].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ROUTEPROTO.fields_by_name[
    'end_time'
].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ROUTEPROTO.fields_by_name['status'].message_type = _STATUSPROTO
_ENVELOPEPROTO_VERSIONPROTO.containing_type = _ENVELOPEPROTO
_ENVELOPEPROTO_COMPRESSCONFIGPROTO.fields_by_name[
    'parameters'
].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_ENVELOPEPROTO_COMPRESSCONFIGPROTO.containing_type = _ENVELOPEPROTO
_ENVELOPEPROTO.fields_by_name['routes'].message_type = _ROUTEPROTO
_ENVELOPEPROTO.fields_by_name['version'].message_type = _ENVELOPEPROTO_VERSIONPROTO
_ENVELOPEPROTO.fields_by_name['status'].message_type = _STATUSPROTO
_ENVELOPEPROTO.fields_by_name[
    'compression'
].message_type = _ENVELOPEPROTO_COMPRESSCONFIGPROTO
_STATUSPROTO_EXCEPTIONPROTO.containing_type = _STATUSPROTO
_STATUSPROTO.fields_by_name['code'].enum_type = _STATUSPROTO_STATUSCODE
_STATUSPROTO.fields_by_name['exception'].message_type = _STATUSPROTO_EXCEPTIONPROTO
_STATUSPROTO_STATUSCODE.containing_type = _STATUSPROTO
_MESSAGEPROTO.fields_by_name['envelope'].message_type = _ENVELOPEPROTO
_MESSAGEPROTO.fields_by_name['request'].message_type = _REQUESTPROTO
_REQUESTPROTO_TRAINREQUESTPROTO.fields_by_name['docs'].message_type = _DOCUMENTPROTO
_REQUESTPROTO_TRAINREQUESTPROTO.fields_by_name[
    'groundtruths'
].message_type = _DOCUMENTPROTO
_REQUESTPROTO_TRAINREQUESTPROTO.containing_type = _REQUESTPROTO
_REQUESTPROTO_INDEXREQUESTPROTO.fields_by_name['docs'].message_type = _DOCUMENTPROTO
_REQUESTPROTO_INDEXREQUESTPROTO.fields_by_name[
    'groundtruths'
].message_type = _DOCUMENTPROTO
_REQUESTPROTO_INDEXREQUESTPROTO.containing_type = _REQUESTPROTO
_REQUESTPROTO_UPDATEREQUESTPROTO.fields_by_name['docs'].message_type = _DOCUMENTPROTO
_REQUESTPROTO_UPDATEREQUESTPROTO.containing_type = _REQUESTPROTO
_REQUESTPROTO_DELETEREQUESTPROTO.containing_type = _REQUESTPROTO
_REQUESTPROTO_SEARCHREQUESTPROTO.fields_by_name['docs'].message_type = _DOCUMENTPROTO
_REQUESTPROTO_SEARCHREQUESTPROTO.fields_by_name[
    'groundtruths'
].message_type = _DOCUMENTPROTO
_REQUESTPROTO_SEARCHREQUESTPROTO.containing_type = _REQUESTPROTO
_REQUESTPROTO_CONTROLREQUESTPROTO.fields_by_name[
    'command'
].enum_type = _REQUESTPROTO_CONTROLREQUESTPROTO_COMMAND
_REQUESTPROTO_CONTROLREQUESTPROTO.fields_by_name[
    'args'
].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_REQUESTPROTO_CONTROLREQUESTPROTO.containing_type = _REQUESTPROTO
_REQUESTPROTO_CONTROLREQUESTPROTO_COMMAND.containing_type = (
    _REQUESTPROTO_CONTROLREQUESTPROTO
)
_REQUESTPROTO.fields_by_name['train'].message_type = _REQUESTPROTO_TRAINREQUESTPROTO
_REQUESTPROTO.fields_by_name['index'].message_type = _REQUESTPROTO_INDEXREQUESTPROTO
_REQUESTPROTO.fields_by_name['search'].message_type = _REQUESTPROTO_SEARCHREQUESTPROTO
_REQUESTPROTO.fields_by_name['control'].message_type = _REQUESTPROTO_CONTROLREQUESTPROTO
_REQUESTPROTO.fields_by_name['delete'].message_type = _REQUESTPROTO_DELETEREQUESTPROTO
_REQUESTPROTO.fields_by_name['update'].message_type = _REQUESTPROTO_UPDATEREQUESTPROTO
_REQUESTPROTO.fields_by_name['status'].message_type = _STATUSPROTO
_REQUESTPROTO.fields_by_name['routes'].message_type = _ROUTEPROTO
_REQUESTPROTO.fields_by_name['queryset'].message_type = _QUERYLANGPROTO
_REQUESTPROTO.oneofs_by_name['body'].fields.append(
    _REQUESTPROTO.fields_by_name['train']
)
_REQUESTPROTO.fields_by_name['train'].containing_oneof = _REQUESTPROTO.oneofs_by_name[
    'body'
]
_REQUESTPROTO.oneofs_by_name['body'].fields.append(
    _REQUESTPROTO.fields_by_name['index']
)
_REQUESTPROTO.fields_by_name['index'].containing_oneof = _REQUESTPROTO.oneofs_by_name[
    'body'
]
_REQUESTPROTO.oneofs_by_name['body'].fields.append(
    _REQUESTPROTO.fields_by_name['search']
)
_REQUESTPROTO.fields_by_name['search'].containing_oneof = _REQUESTPROTO.oneofs_by_name[
    'body'
]
_REQUESTPROTO.oneofs_by_name['body'].fields.append(
    _REQUESTPROTO.fields_by_name['control']
)
_REQUESTPROTO.fields_by_name['control'].containing_oneof = _REQUESTPROTO.oneofs_by_name[
    'body'
]
_REQUESTPROTO.oneofs_by_name['body'].fields.append(
    _REQUESTPROTO.fields_by_name['delete']
)
_REQUESTPROTO.fields_by_name['delete'].containing_oneof = _REQUESTPROTO.oneofs_by_name[
    'body'
]
_REQUESTPROTO.oneofs_by_name['body'].fields.append(
    _REQUESTPROTO.fields_by_name['update']
)
_REQUESTPROTO.fields_by_name['update'].containing_oneof = _REQUESTPROTO.oneofs_by_name[
    'body'
]
_QUERYLANGPROTO.fields_by_name[
    'parameters'
].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
DESCRIPTOR.message_types_by_name['DenseNdArrayProto'] = _DENSENDARRAYPROTO
DESCRIPTOR.message_types_by_name['NdArrayProto'] = _NDARRAYPROTO
DESCRIPTOR.message_types_by_name['SparseNdArrayProto'] = _SPARSENDARRAYPROTO
DESCRIPTOR.message_types_by_name['NamedScoreProto'] = _NAMEDSCOREPROTO
DESCRIPTOR.message_types_by_name['DocumentProto'] = _DOCUMENTPROTO
DESCRIPTOR.message_types_by_name['RouteProto'] = _ROUTEPROTO
DESCRIPTOR.message_types_by_name['EnvelopeProto'] = _ENVELOPEPROTO
DESCRIPTOR.message_types_by_name['StatusProto'] = _STATUSPROTO
DESCRIPTOR.message_types_by_name['MessageProto'] = _MESSAGEPROTO
DESCRIPTOR.message_types_by_name['RequestProto'] = _REQUESTPROTO
DESCRIPTOR.message_types_by_name['QueryLangProto'] = _QUERYLANGPROTO
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DenseNdArrayProto = _reflection.GeneratedProtocolMessageType(
    'DenseNdArrayProto',
    (_message.Message,),
    {
        'DESCRIPTOR': _DENSENDARRAYPROTO,
        '__module__': 'jina_pb2'
        # @@protoc_insertion_point(class_scope:jina.DenseNdArrayProto)
    },
)
_sym_db.RegisterMessage(DenseNdArrayProto)

NdArrayProto = _reflection.GeneratedProtocolMessageType(
    'NdArrayProto',
    (_message.Message,),
    {
        'DESCRIPTOR': _NDARRAYPROTO,
        '__module__': 'jina_pb2'
        # @@protoc_insertion_point(class_scope:jina.NdArrayProto)
    },
)
_sym_db.RegisterMessage(NdArrayProto)

SparseNdArrayProto = _reflection.GeneratedProtocolMessageType(
    'SparseNdArrayProto',
    (_message.Message,),
    {
        'DESCRIPTOR': _SPARSENDARRAYPROTO,
        '__module__': 'jina_pb2'
        # @@protoc_insertion_point(class_scope:jina.SparseNdArrayProto)
    },
)
_sym_db.RegisterMessage(SparseNdArrayProto)

NamedScoreProto = _reflection.GeneratedProtocolMessageType(
    'NamedScoreProto',
    (_message.Message,),
    {
        'DESCRIPTOR': _NAMEDSCOREPROTO,
        '__module__': 'jina_pb2'
        # @@protoc_insertion_point(class_scope:jina.NamedScoreProto)
    },
)
_sym_db.RegisterMessage(NamedScoreProto)

DocumentProto = _reflection.GeneratedProtocolMessageType(
    'DocumentProto',
    (_message.Message,),
    {
        'DESCRIPTOR': _DOCUMENTPROTO,
        '__module__': 'jina_pb2'
        # @@protoc_insertion_point(class_scope:jina.DocumentProto)
    },
)
_sym_db.RegisterMessage(DocumentProto)

RouteProto = _reflection.GeneratedProtocolMessageType(
    'RouteProto',
    (_message.Message,),
    {
        'DESCRIPTOR': _ROUTEPROTO,
        '__module__': 'jina_pb2'
        # @@protoc_insertion_point(class_scope:jina.RouteProto)
    },
)
_sym_db.RegisterMessage(RouteProto)

EnvelopeProto = _reflection.GeneratedProtocolMessageType(
    'EnvelopeProto',
    (_message.Message,),
    {
        'VersionProto': _reflection.GeneratedProtocolMessageType(
            'VersionProto',
            (_message.Message,),
            {
                'DESCRIPTOR': _ENVELOPEPROTO_VERSIONPROTO,
                '__module__': 'jina_pb2'
                # @@protoc_insertion_point(class_scope:jina.EnvelopeProto.VersionProto)
            },
        ),
        'CompressConfigProto': _reflection.GeneratedProtocolMessageType(
            'CompressConfigProto',
            (_message.Message,),
            {
                'DESCRIPTOR': _ENVELOPEPROTO_COMPRESSCONFIGPROTO,
                '__module__': 'jina_pb2'
                # @@protoc_insertion_point(class_scope:jina.EnvelopeProto.CompressConfigProto)
            },
        ),
        'DESCRIPTOR': _ENVELOPEPROTO,
        '__module__': 'jina_pb2'
        # @@protoc_insertion_point(class_scope:jina.EnvelopeProto)
    },
)
_sym_db.RegisterMessage(EnvelopeProto)
_sym_db.RegisterMessage(EnvelopeProto.VersionProto)
_sym_db.RegisterMessage(EnvelopeProto.CompressConfigProto)

StatusProto = _reflection.GeneratedProtocolMessageType(
    'StatusProto',
    (_message.Message,),
    {
        'ExceptionProto': _reflection.GeneratedProtocolMessageType(
            'ExceptionProto',
            (_message.Message,),
            {
                'DESCRIPTOR': _STATUSPROTO_EXCEPTIONPROTO,
                '__module__': 'jina_pb2'
                # @@protoc_insertion_point(class_scope:jina.StatusProto.ExceptionProto)
            },
        ),
        'DESCRIPTOR': _STATUSPROTO,
        '__module__': 'jina_pb2'
        # @@protoc_insertion_point(class_scope:jina.StatusProto)
    },
)
_sym_db.RegisterMessage(StatusProto)
_sym_db.RegisterMessage(StatusProto.ExceptionProto)

MessageProto = _reflection.GeneratedProtocolMessageType(
    'MessageProto',
    (_message.Message,),
    {
        'DESCRIPTOR': _MESSAGEPROTO,
        '__module__': 'jina_pb2'
        # @@protoc_insertion_point(class_scope:jina.MessageProto)
    },
)
_sym_db.RegisterMessage(MessageProto)

RequestProto = _reflection.GeneratedProtocolMessageType(
    'RequestProto',
    (_message.Message,),
    {
        'TrainRequestProto': _reflection.GeneratedProtocolMessageType(
            'TrainRequestProto',
            (_message.Message,),
            {
                'DESCRIPTOR': _REQUESTPROTO_TRAINREQUESTPROTO,
                '__module__': 'jina_pb2'
                # @@protoc_insertion_point(class_scope:jina.RequestProto.TrainRequestProto)
            },
        ),
        'IndexRequestProto': _reflection.GeneratedProtocolMessageType(
            'IndexRequestProto',
            (_message.Message,),
            {
                'DESCRIPTOR': _REQUESTPROTO_INDEXREQUESTPROTO,
                '__module__': 'jina_pb2'
                # @@protoc_insertion_point(class_scope:jina.RequestProto.IndexRequestProto)
            },
        ),
        'UpdateRequestProto': _reflection.GeneratedProtocolMessageType(
            'UpdateRequestProto',
            (_message.Message,),
            {
                'DESCRIPTOR': _REQUESTPROTO_UPDATEREQUESTPROTO,
                '__module__': 'jina_pb2'
                # @@protoc_insertion_point(class_scope:jina.RequestProto.UpdateRequestProto)
            },
        ),
        'DeleteRequestProto': _reflection.GeneratedProtocolMessageType(
            'DeleteRequestProto',
            (_message.Message,),
            {
                'DESCRIPTOR': _REQUESTPROTO_DELETEREQUESTPROTO,
                '__module__': 'jina_pb2'
                # @@protoc_insertion_point(class_scope:jina.RequestProto.DeleteRequestProto)
            },
        ),
        'SearchRequestProto': _reflection.GeneratedProtocolMessageType(
            'SearchRequestProto',
            (_message.Message,),
            {
                'DESCRIPTOR': _REQUESTPROTO_SEARCHREQUESTPROTO,
                '__module__': 'jina_pb2'
                # @@protoc_insertion_point(class_scope:jina.RequestProto.SearchRequestProto)
            },
        ),
        'ControlRequestProto': _reflection.GeneratedProtocolMessageType(
            'ControlRequestProto',
            (_message.Message,),
            {
                'DESCRIPTOR': _REQUESTPROTO_CONTROLREQUESTPROTO,
                '__module__': 'jina_pb2'
                # @@protoc_insertion_point(class_scope:jina.RequestProto.ControlRequestProto)
            },
        ),
        'DESCRIPTOR': _REQUESTPROTO,
        '__module__': 'jina_pb2'
        # @@protoc_insertion_point(class_scope:jina.RequestProto)
    },
)
_sym_db.RegisterMessage(RequestProto)
_sym_db.RegisterMessage(RequestProto.TrainRequestProto)
_sym_db.RegisterMessage(RequestProto.IndexRequestProto)
_sym_db.RegisterMessage(RequestProto.UpdateRequestProto)
_sym_db.RegisterMessage(RequestProto.DeleteRequestProto)
_sym_db.RegisterMessage(RequestProto.SearchRequestProto)
_sym_db.RegisterMessage(RequestProto.ControlRequestProto)

QueryLangProto = _reflection.GeneratedProtocolMessageType(
    'QueryLangProto',
    (_message.Message,),
    {
        'DESCRIPTOR': _QUERYLANGPROTO,
        '__module__': 'jina_pb2'
        # @@protoc_insertion_point(class_scope:jina.QueryLangProto)
    },
)
_sym_db.RegisterMessage(QueryLangProto)


_JINARPC = _descriptor.ServiceDescriptor(
    name='JinaRPC',
    full_name='jina.JinaRPC',
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=3567,
    serialized_end=3630,
    methods=[
        _descriptor.MethodDescriptor(
            name='Call',
            full_name='jina.JinaRPC.Call',
            index=0,
            containing_service=None,
            input_type=_REQUESTPROTO,
            output_type=_REQUESTPROTO,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_JINARPC)

DESCRIPTOR.services_by_name['JinaRPC'] = _JINARPC

# @@protoc_insertion_point(module_scope)



from typing import Dict, List

from jina.executors.segmenters import BaseSegmenter


class JiebaSegmenter(BaseSegmenter):
    """
     :class:`JiebaSegmenter` split the chinese text
     on the doc-level into words on the chunk-level with `jieba`.
     :param mode: Jieba mode. The default mode is 'accurate'
     Options are:
     - accurate
     - search
     - all

    :type mode: str
    :param user_dict_file: Path to a custom dictionary.
        This custom dictionary extends the default dictionary.
        `See here for the format of the dict https://github.com/fxsjy/jieba#load-dictionary`_
    :type user_dict_file: str
    :raises:
        ValueError: If `mode` is not any of the expected modes
    :param args:  Additional positional arguments
    :param kwargs: Additional keyword arguments

    """

    def __init__(self, mode: str = 'accurate', user_dict_file: str = None, *args, **kwargs):
        """Set Constructor."""
        super().__init__(*args, **kwargs)
        if mode not in ('accurate', 'all', 'search'):
            raise ValueError('you must choose one of modes to cut the text: accurate, all, search.')
        self.mode = mode
        self.user_dict_file = user_dict_file

    def post_init(self):
        """ Load custom dict if provided. Raise FileNotFoundError if dict does not exist."""
        super().post_init()

        if self.user_dict_file is not None:
            import os
            import jieba
            if not os.path.exists(self.user_dict_file):
                raise FileNotFoundError(f'User dictionary can not be found at {self.user_dict_file}')
            jieba.load_userdict(self.user_dict_file)

    def segment(self, text: str, *args, **kwargs) -> List[Dict]:
        """
        Split the chinese text into words.

        :param text: Raw text to be segmented
        :type text: str
        :param args:  Additional positional arguments
        :param kwargs: Additional keyword arguments
        :return: Sub-documents segmented
        :rtype: List[Dict]
        """
        import jieba
        if self.mode == 'search':
            words = jieba.cut_for_search(text)
        elif self.mode == 'all':
            words = jieba.cut(text, cut_all=True)
        else:
            words = jieba.cut(text)

        chunks = []
        for idx, word in enumerate(words):
            chunks.append(
                dict(text=word, offset=idx, weight=1.0))

        return chunks



__copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
__license__ = "Apache-2.0"

from typing import Optional, Tuple

from . import BaseExecutableDriver, FlatRecursiveMixin
from ..excepts import LengthMismatchException
from ..types.document import Document

if False:
    from .. import DocumentSet


class SegmentDriver(FlatRecursiveMixin, BaseExecutableDriver):
    """Drivers inherited from this Driver will bind :meth:`segment` by default """

    def __init__(
        self,
        executor: Optional[str] = None,
        method: str = 'segment',
        traversal_paths: Tuple[str] = ('r',),
        *args,
        **kwargs,
    ):
        super().__init__(
            executor, method, traversal_paths=traversal_paths, *args, **kwargs
        )

    def _apply_all(self, docs: 'DocumentSet', *args, **kwargs):
        if not docs:
            self.logger.warning(f'an empty DocumentSet {docs}')
            return

        contents, docs_pts = docs.extract_docs(*self.exec.required_keys)

        if not docs_pts:
            self.logger.warning(f'no Document is extracted {docs}')
            return

        if len(self.exec.required_keys) > 1:
            docs_chunks = self.exec_fn(*contents)
        else:
            docs_chunks = self.exec_fn(contents)

        if len(docs_pts) != len(docs_chunks):
            msg = (
                f'mismatched {len(docs_pts)} docs from level {docs_pts[0].granularity} '
                f'and length of returned crafted documents: {len(docs_chunks)}, the length must be the same'
            )
            self.logger.error(msg)
            raise LengthMismatchException(msg)

        for doc, chunks in zip(docs_pts, docs_chunks):
            self._add_chunks(doc, chunks)

    @staticmethod
    def _add_chunks(doc, chunks):
        num_siblings = len(chunks)
        for chunk in chunks:
            with Document(length=num_siblings, **chunk) as c:
                if not c.mime_type:
                    c.mime_type = doc.mime_type
            doc.chunks.append(c)



"""Indexer for caching."""

import pickle
import tempfile
from typing import Optional, Iterable, List, Tuple, Union

from jina.executors.indexers import BaseKVIndexer
from jina.helper import deprecated_alias

DATA_FIELD = 'data'
ID_KEY = 'id'
CONTENT_HASH_KEY = 'content_hash'


class BaseCache(BaseKVIndexer):
    """Base class of the cache inherited :class:`BaseKVIndexer`.

    The difference between a cache and a :class:`BaseKVIndexer` is the ``handler_mutex`` is released in cache,
    this allows one to query-while-indexing.

    :param args: additional positional arguments wich are just used for the parent initialization
    :param kwargs: additional key value arguments wich are just used for the parent initialization
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def post_init(self):
        """For Cache we need to release the handler mutex to allow RW at the same time."""
        self.handler_mutex = False


class DocCache(BaseCache):
    """A key-value indexer that specializes in caching.

    Serializes the cache to two files, one for ids, one for the actually cached field.
    If fields=["id"], then the second file is redundant. The class optimizes the process
    so that there are no duplicates.

    Order of fields does NOT affect the caching.

    :param index_filename: file name for storing the cache data
    :param fields: fields to cache on (of Document)
    :param args: additional positional arguments wich are just used for the parent initialization
    :param kwargs: additional key value arguments wich are just used for the parent initialization
    """

    class CacheHandler:
        """A handler for loading and serializing the in-memory cache of the DocCache.

        :param path: Path to the file from which to build the actual paths.
        :param logger: Instance of logger.
        """

        def __init__(self, path, logger):
            self.path = path
            try:
                self.id_to_cache_val = pickle.load(open(path + '.ids', 'rb'))
                self.cache_val_to_id = pickle.load(open(path + '.cache', 'rb'))
            except FileNotFoundError as e:
                logger.warning(
                    f'File path did not exist : {path}.ids or {path}.cache: {e!r}. Creating new CacheHandler...'
                )
                self.id_to_cache_val = dict()
                self.cache_val_to_id = dict()

        def close(self):
            """Flushes the in-memory cache to pickle files."""
            pickle.dump(self.id_to_cache_val, open(self.path + '.ids', 'wb'))
            pickle.dump(self.cache_val_to_id, open(self.path + '.cache', 'wb'))

    default_fields = (ID_KEY,)

    @deprecated_alias(field=('fields', 0))
    def __init__(
        self,
        index_filename: Optional[str] = None,
        fields: Optional[
            Union[str, Tuple[str]]
        ] = None,  # str for backwards compatibility
        *args,
        **kwargs,
    ):
        if not index_filename:
            # create a new temp file if not exist
            index_filename = tempfile.NamedTemporaryFile(delete=False).name
        super().__init__(index_filename, *args, **kwargs)
        if isinstance(fields, str):
            fields = (fields,)
        # order shouldn't matter
        self.fields = sorted(fields or self.default_fields)

    def add(
        self, keys: Iterable[str], values: Iterable[bytes], *args, **kwargs
    ) -> None:
        """Add a document to the cache depending.

        :param keys: document ids to be added
        :param values: document cache values to be added
        :param args: not used
        :param kwargs: not used
        """
        for key, value in zip(keys, values):
            self.query_handler.id_to_cache_val[key] = value
            self.query_handler.cache_val_to_id[value] = key
            self._size += 1

    def query(self, key: str, *args, **kwargs) -> bool:
        """Check whether the data exists in the cache.

        :param key: the value that we cached by (combination of the Document fields)
        :param args: not used
        :param kwargs: not used
        :return: status
        """
        return key in self.query_handler.cache_val_to_id

    def update(
        self, keys: Iterable[str], values: Iterable[bytes], *args, **kwargs
    ) -> None:
        """Update cached documents.

        :param keys: list of Document.id
        :param values: list of values (combination of the Document fields)
        :param args: not used
        :param kwargs: not used
        """

        if len(self.fields) == 1 and self.fields[0] == ID_KEY:
            # if we don't cache anything else, no need
            return

        for key, value in zip(keys, values):
            if key not in self.query_handler.id_to_cache_val:
                continue
            old_value = self.query_handler.id_to_cache_val[key]
            self.query_handler.id_to_cache_val[key] = value
            del self.query_handler.cache_val_to_id[old_value]
            self.query_handler.cache_val_to_id[value] = key

    def delete(self, keys: Iterable[str], *args, **kwargs) -> None:
        """Delete documents from the cache.

        :param keys: list of Document.id
        :param args: not used
        :param kwargs: not used
        """
        for key in keys:
            if key not in self.query_handler.id_to_cache_val:
                continue
            value = self.query_handler.id_to_cache_val[key]
            del self.query_handler.id_to_cache_val[key]
            del self.query_handler.cache_val_to_id[value]
            self._size -= 1

    def get_add_handler(self):
        """Get the CacheHandler.


        .. # noqa: DAR201"""
        return self.get_query_handler()

    def get_query_handler(self) -> CacheHandler:
        """Get the CacheHandler.


        .. # noqa: DAR201"""
        return self.CacheHandler(self.save_abspath, self.logger)

    def get_create_handler(self):
        """Get the CacheHandler.


        .. # noqa: DAR201"""
        return self.get_query_handler()
